@@ -183,6 +183,49 @@ NamedCallTarget = NamedTuple(
 )
 
 
+def intm_return_targets(intm):
+    if intm.matches.Effect:
+        return intm.expr.returnTargets()
+
+    if intm.matches.Terminal:
+        return intm.expr.returnTargets()
+
+    if intm.matches.Simple:
+        return intm.expr.returnTargets()
+
+    if intm.matches.StackSlot:
+        return intm.expr.returnTargets()
+
+    assert False, f"Unrecognized expression intermediate {intm}"
+
+
+def intm_could_throw(intm):
+    if intm.matches.Effect:
+        return intm.expr.couldThrow()
+
+    if intm.matches.Terminal:
+        return intm.expr.couldThrow()
+
+    if intm.matches.Simple:
+        return intm.expr.couldThrow()
+
+    if intm.matches.StackSlot:
+        return intm.expr.couldThrow()
+
+    assert False, f"Unrecognized expression intermediate {intm}"
+
+
+ExpressionIntermediate = Alternative(
+    "ExpressionIntermediate",
+    Effect={"expr": Expression},
+    Terminal={"expr": Expression},
+    Simple={"name": str, "expr": Expression},
+    StackSlot={"name": str, "expr": Expression},
+    returnTargets=intm_return_targets,
+    couldThrow=intm_could_throw,
+)
+
+
 def filterCallTargetArgs(args):
     """Given a list of native expressions or typed expressions, filter them down,
     dropping 'empty' arguments, as per our calling convention."""
@@ -382,6 +425,10 @@ def expr_str(self):
             "try:\n" + indent(str(self.expr)) + "\n" +
             "rethrow with %s:\n" % self.varname + indent(str(self.handler)).rstrip()
         )
+    if self.matches.ApplyIntermediates:
+        return (
+            "apply intermediates:\n" + "\n".join(str(x) for x in self.intermediates) + "\nto " + str(self.base)
+        )
 
     assert False, type(self)
 
@@ -399,6 +446,8 @@ def expr_is_simple(expr):
         return expr_is_simple(expr.ptr)
     if expr.matches.Sequence and len(expr.vals) == 1:
         return expr_is_simple(expr.vals[0])
+    if expr.matches.ApplyIntermediates and len(expr.intermediates) == 0:
+        return expr_is_simple(expr.base)
     return False
 
 
@@ -428,6 +477,9 @@ def expr_return_targets(expr):
         elif isinstance(child, TupleOf(Expression)):
             for c in child:
                 res |= c.returnTargets()
+        elif isinstance(child, TupleOf(ExpressionIntermediate)):
+            for c in child:
+                res |= c.returnTargets()
 
     return res
 
@@ -461,6 +513,10 @@ def expr_could_throw(self):
             for c in child:
                 if c.couldThrow():
                     return True
+        elif isinstance(child, TupleOf(ExpressionIntermediate)):
+            for c in child:
+                if c.couldThrow():
+                    return True
 
     return False
 
@@ -521,6 +577,7 @@ Expression = Expression.define(Alternative(
     ActivatesTeardown={'name': str},
     StackSlot={'name': str, 'type': Type},
     GlobalVariable={'name': str, 'type': Type, 'metadata': object},
+    ApplyIntermediates={'base': Expression, 'intermediates': TupleOf(ExpressionIntermediate)},
     ElementPtrIntegers=lambda self, *offsets:
         Expression.ElementPtr(
             left=self,
