@@ -149,6 +149,9 @@ class PythonToNativeConverter:
 
         self._dependencies = FunctionDependencyGraph()
 
+    def getDefinitionCount(self):
+        return len(self._definitions)
+
     def addVisitor(self, visitor):
         self._visitors.append(visitor)
 
@@ -246,7 +249,11 @@ class PythonToNativeConverter:
 
     def defineLinkName(self, identity, linkName):
         if identity in self._link_name_for_identity:
-            assert self._link_name_for_identity[identity] == linkName
+            if self._link_name_for_identity[identity] != linkName:
+                raise Exception(
+                    f"For identity {identity}:\n\n"
+                    f"{self._link_name_for_identity[identity]}\n\n!=\n\n{linkName}"
+                )
             assert self._identity_for_link_name[linkName] == identity
         else:
             self._link_name_for_identity[identity] = linkName
@@ -304,7 +311,8 @@ class PythonToNativeConverter:
                 name,
                 context.getInputTypes(),
                 context.knownOutputType(),
-                alwaysRaises=context.alwaysRaises()
+                alwaysRaises=context.alwaysRaises(),
+                functionMetadata=context.functionMetadata
             )
 
         if self._currentlyConverting is None:
@@ -348,7 +356,7 @@ class PythonToNativeConverter:
             )
         )
 
-    def getTypedCallTarget(self, name, input_types, output_type, alwaysRaises=False):
+    def getTypedCallTarget(self, name, input_types, output_type, alwaysRaises=False, functionMetadata=None):
         native_input_types = [a.getNativePassingType() for a in input_types if not a.is_empty]
         if output_type is None:
             native_output_type = native_ast.Type.Void()
@@ -370,7 +378,8 @@ class PythonToNativeConverter:
             ),
             input_types,
             output_type,
-            alwaysRaises=alwaysRaises
+            alwaysRaises=alwaysRaises,
+            functionMetadata=functionMetadata
         )
 
         return res
@@ -544,7 +553,8 @@ class PythonToNativeConverter:
                     name,
                     functionConverter._input_types,
                     actual_output_type,
-                    functionConverter.alwaysRaises()
+                    alwaysRaises=functionConverter.alwaysRaises(),
+                    functionMetadata=functionConverter.functionMetadata
                 )
 
             if dirtyUpstream:
@@ -832,8 +842,13 @@ class PythonToNativeConverter:
                         self._identifier_to_pyfunc[identifier]
                     )
 
+                    nativeFunction, actual_output_type = self._inflight_definitions.get(identifier)
+
                     try:
                         v.onNewFunction(
+                            identifier,
+                            functionConverter,
+                            nativeFunction,
                             funcName,
                             funcCode,
                             funcGlobals,
